"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const util_1 = require("./util");
const bluebird_1 = require("bluebird");
const mkdirp = require("mkdirp");
const mkdirpAsync = bluebird_1.promisify(mkdirp);
const unlinkAsync = bluebird_1.promisify(fs_1.unlink);
const readdirAsync = bluebird_1.promisify(fs_1.readdir);
function readDirAsync(dir) {
    return readdirAsync(dir)
        .map((file) => {
        const path = path_1.join(dir, file);
        return util_1.isDirectoryAsync(path).then((x) => (x ? readDirAsync(path) : path));
    })
        .reduce((a, b) => a.concat(b), []);
}
function maybeReadFileContentsAsync(file) {
    return util_1.readFileAsync(file, 'utf-8').catch(e => {
        if (e.code === 'ENOENT') {
            return '';
        }
        throw e;
    });
}
/**
 * The artifacts step is where source patches are committed, or written as "artifacts"
 * Steps:
 *  - A temporary directory is created in the downloaded source
 *  - On start, any files in that directory are restored into the source tree
 *  - After the patch functions have run, the temporary directory is emptied
 *  - Original versions of sources to be patched are written to the temporary directory
 *  - Finally, The patched files are written into source.
 *
 */
function artifacts(compiler, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const { src } = compiler;
        const temp = path_1.join(src, 'nexe');
        yield mkdirpAsync(temp);
        const tmpFiles = yield readDirAsync(temp);
        yield bluebird_1.map(tmpFiles, (path) => __awaiter(this, void 0, void 0, function* () {
            return compiler.writeFileAsync(path.replace(temp, ''), yield util_1.readFileAsync(path, 'utf-8'));
        }));
        yield next();
        yield bluebird_1.map(tmpFiles, x => unlinkAsync(x));
        return bluebird_1.map(compiler.files, (file) => __awaiter(this, void 0, void 0, function* () {
            const sourceFile = path_1.join(src, file.filename);
            const tempFile = path_1.join(temp, file.filename);
            const fileContents = yield maybeReadFileContentsAsync(sourceFile);
            yield mkdirpAsync(path_1.dirname(tempFile));
            yield util_1.writeFileAsync(tempFile, fileContents);
            yield compiler.writeFileAsync(file.filename, file.contents);
        }));
    });
}
exports.default = artifacts;
