/// <reference types="node" />
import { Readable } from 'stream';
import { Logger } from './logger';
import { NexeOptions } from './options';
export interface NexeFile {
    filename: string;
    contents: string;
}
export declare class NexeCompiler {
    options: NexeOptions;
    private start;
    private env;
    private compileStep;
    log: Logger;
    src: string;
    files: NexeFile[];
    input: string;
    output: string | null;
    resources: {
        bundle: string;
        index: {
            [key: string]: number[];
        };
    };
    readFileAsync: (file: string) => Promise<NexeFile>;
    writeFileAsync: (file: string, contents: Buffer | string) => Promise<void>;
    replaceInFileAsync: (file: string, replacer: any, replaceValue: string) => Promise<void>;
    setFileContentsAsync: (file: string, contents: string | Buffer) => Promise<void>;
    private nodeSrcBinPath;
    constructor(options: NexeOptions);
    quit(code?: number): Promise<void>;
    private _findPaddingSize(size, override?);
    private _getNodeExecutableLocation(target?);
    private _runBuildCommandAsync(command, args);
    private _configureAsync();
    private _buildAsync();
    private _fetchPrebuiltBinaryAsync();
    private _getPayload(header);
    private _generateHeader();
    private _getExistingBinaryHeaderAsync(target);
    private _extractHeaderAsync(path);
    private _serializeHeader(header);
    setMainModule(compiler: NexeCompiler, next: () => Promise<void>): Promise<void>;
    compileAsync(): Promise<Readable>;
    private _assembleDeliverable(header, binary);
}
