"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const download = require("download");
const util_1 = require("./util");
function fetchNodeSourceAsync(cwd, url, step, options = {}) {
    const setText = (p) => step.modify(`Downloading Node: ${p.toFixed()}%...`);
    return download(url, cwd, Object.assign(options, { extract: true, strip: 1 }))
        .on('response', (res) => {
        const total = +res.headers['content-length'];
        let current = 0;
        res.on('data', data => {
            current += data.length;
            setText(current / total * 100);
            if (current === total) {
                step.log('Extracting Node...');
            }
        });
    })
        .then(() => step.log(`Node source extracted to: ${cwd}`));
}
/**
 * Downloads the node source to the configured temporary directory
 * @param {*} compiler
 * @param {*} next
 */
function downloadNode(compiler, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const { src, log } = compiler;
        const { version, sourceUrl, downloadOptions } = compiler.options;
        const url = sourceUrl || `https://nodejs.org/dist/v${version}/node-v${version}.tar.gz`;
        const step = log.step(`Downloading Node.js source from: ${url}`);
        if (yield util_1.isDirectoryAsync(src)) {
            step.log('Source already downloaded');
            return next();
        }
        return fetchNodeSourceAsync(src, url, step, downloadOptions).then(next);
    });
}
exports.default = downloadNode;
