"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const app_builder_1 = require("app-builder");
const resource_1 = require("./resource");
const compiler_1 = require("./compiler");
const options_1 = require("./options");
exports.argv = options_1.argv;
const cli_1 = require("./cli");
const download_1 = require("./download");
const artifacts_1 = require("./artifacts");
const patches_1 = require("./patches");
const util_1 = require("./util");
const Bluebird = require("bluebird");
app_builder_1.PromiseConfig.constructor = Bluebird;
function compile(compilerOptions, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = yield options_1.normalizeOptionsAsync(compilerOptions);
        const compiler = new compiler_1.NexeCompiler(options);
        const build = compiler.options.build;
        if (options.clean) {
            const step = compiler.log.step('Cleaning up nexe build artifacts...');
            step.log(`Deleting directory and contents at: ${compiler.src}`);
            yield util_1.rimrafAsync(compiler.src);
            step.log(`Deleted directory and contents at: ${compiler.src}`);
            return compiler.quit();
        }
        const buildSteps = build ? [download_1.default, artifacts_1.default, ...patches_1.default, ...options.patches] : [];
        const nexe = app_builder_1.compose(resource_1.default, cli_1.default, buildSteps);
        return nexe(compiler).asCallback(callback);
    });
}
exports.compile = compile;
