/// <reference types="minimist" />
import * as parseArgv from 'minimist';
import { NexeCompiler } from './compiler';
export interface NexePatch {
    (compiler: NexeCompiler, next: () => Promise<void>): Promise<void>;
}
export interface NexeOptions {
    build: boolean;
    /**
     * Entrypoint filepath
     */
    input: string;
    output: string;
    targets: string[];
    /**
     * Build name, Used for executable, and stacktraces
     */
    name: string;
    /**
     * The node version to be built
     */
    version: string;
    python?: string;
    /**
     * Node flags e.g. "--expose-gc" baked into the executable
     */
    flags: string[];
    /**
     * Pass configuration options to node build configure script
     */
    configure: string[];
    /**
     * Pass make options to node make script
     */
    make: string[];
    vcBuild: string[];
    snapshot?: string;
    /**
     * Array of glob strings describing resources to pull into the bundle
     */
    resources: string[];
    /**
     * Temporary directory where nexe artifacts will be cached
     * TODO dot folder in home dir
     */
    temp: string;
    ico?: string;
    rc: {
        [key: string]: string;
    };
    /**
     * Causes nexe to remove all temporary files for current configuration
     */
    clean: boolean;
    enableNodeCli: boolean;
    sourceUrl?: string;
    loglevel: 'info' | 'silent' | 'verbose';
    silent?: boolean;
    verbose?: boolean;
    info?: boolean;
    padding: number;
    patches: NexePatch[];
    empty: boolean;
    warmup?: string;
    downloadOptions?: any;
}
declare const argv: parseArgv.ParsedArgs;
declare function normalizeOptionsAsync(input: Partial<NexeOptions>): Promise<NexeOptions>;
export { argv, normalizeOptionsAsync };
