"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = require("bluebird");
const util_1 = require("./util");
const buffer_1 = require("buffer");
const globs = require("globby");
function resource(compiler, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const resources = compiler.resources;
        if (!compiler.options.resources.length) {
            return next();
        }
        const step = compiler.log.step('Bundling Resources...');
        let count = 0;
        yield bluebird_1.each(globs(compiler.options.resources), (file) => __awaiter(this, void 0, void 0, function* () {
            if (yield util_1.isDirectoryAsync(file)) {
                return;
            }
            count++;
            step.log(`Including file: ${file}`);
            const contents = yield util_1.readFileAsync(file);
            const commentSafeContents = contents.toString('base64');
            resources.index[file] = [
                buffer_1.Buffer.byteLength(resources.bundle),
                buffer_1.Buffer.byteLength(commentSafeContents)
            ];
            resources.bundle += commentSafeContents;
        }));
        step.log(`Included ${count} file(s). ${(buffer_1.Buffer.byteLength(resources.bundle) / 1e6).toFixed(3)} MB`);
        return next();
    });
}
exports.default = resource;
